@@ -6,7 +6,7 @@
  *      SPARC AUDIO .AU-files
  *      Raw Data
  *
- *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) by Jaroslav Kysela <perex@perex.cz>
  *  Based on vplay program by Michael Beck
  *
  *
@@ -37,6 +37,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
+#include <time.h>
 #include <locale.h>
 #include <alsa/asoundlib.h>
 #include <assert.h>
@@ -80,7 +81,6 @@ static struct {
 static int timelimit = 0;
 static int quiet_mode = 0;
 static int file_type = FORMAT_DEFAULT;
-static unsigned int sleep_min = 0;
 static int open_mode = 0;
 static snd_pcm_stream_t stream = SND_PCM_STREAM_PLAYBACK;
 static int mmap_flag = 0;
@@ -155,7 +155,7 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "-h, --help              help\n"
 "    --version           print current version\n"
 "-l, --list-devices      list all soundcards and digital audio devices\n"
-"-L, --list-pcms         list all PCMs defined\n"
+"-L, --list-pcms         list device names\n"
 "-D, --device=NAME       select PCM by name\n"
 "-q, --quiet             quiet mode\n"
 "-t, --file-type TYPE    file type (voc, wav, raw or au)\n"
@@ -163,7 +163,6 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "-f, --format=FORMAT     sample format (case insensitive)\n"
 "-r, --rate=#            sample rate\n"
 "-d, --duration=#        interrupt after # seconds\n"
-"-s, --sleep-min=#       min ticks to sleep\n"
 "-M, --mmap              mmap stream\n"
 "-N, --nonblock          nonblocking mode\n"
 "-F, --period-time=#     distance between interrupts is # microseconds\n"
@@ -175,7 +174,11 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "                        (relative to buffer size if <= 0)\n"
 "-T, --stop-delay=#      delay for automatic PCM stop is # microseconds from xrun\n"
 "-v, --verbose           show PCM structure and setup (accumulative)\n"
-"-I, --separate-channels one file for each channel\n")
+"-I, --separate-channels one file for each channel\n"
+"    --disable-resample  disable automatic rate resample\n"
+"    --disable-channels  disable automatic channel conversions\n"
+"    --disable-format    disable automatic format conversions\n"
+"    --disable-softvol   disable software volume control (softvol)\n")
 		, command);
 	printf(_("Recognized sample formats are:"));
 	for (k = 0; k < SND_PCM_FORMAT_LAST; ++k) {
@@ -190,24 +193,6 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 	printf(_("-f dat (16 bit little endian, 48000, stereo)\n"));
 }
 
-static void names_list(void)
-{
-	int err;
-	snd_devname_t *list, *item;
-
-	err = snd_names_list("pcm", &list);
-	if (err < 0) {
-		error(_("snd_names_list error: %s"), snd_strerror(err));
-		return;
-	}
-	item = list;
-	while (item) {
-		printf("%s [%s]\n", item->name, item->comment);
-		item = item->next;
-	}
-	snd_names_list_free(list);
-}
-
 static void device_list(void)
 {
 	snd_ctl_t *handle;
@@ -280,26 +265,47 @@ static void device_list(void)
 
 static void pcm_list(void)
 {
-	snd_config_t *conf;
-	snd_output_t *out;
-	int err = snd_config_update();
-	if (err < 0) {
-		error("snd_config_update: %s", snd_strerror(err));
+	void **hints, **n;
+	char *name, *descr, *descr1, *io;
+	const char *filter;
+
+	if (snd_device_name_hint(-1, "pcm", &hints) < 0)
 		return;
+	n = hints;
+	filter = stream == SND_PCM_STREAM_CAPTURE ? "Input" : "Output";
+	while (*n != NULL) {
+		name = snd_device_name_get_hint(*n, "NAME");
+		descr = snd_device_name_get_hint(*n, "DESC");
+		io = snd_device_name_get_hint(*n, "IOID");
+		if (io != NULL && strcmp(io, filter) == 0)
+			goto __end;
+		printf("%s\n", name);
+		if ((descr1 = descr) != NULL) {
+			printf("    ");
+			while (*descr1) {
+				if (*descr1 == '\n')
+					printf("\n    ");
+				else
+					putchar(*descr1);
+				descr1++;
+			}
+			putchar('\n');
+		}
+	      __end:
+	      	if (name != NULL)
+	      		free(name);
+		if (descr != NULL)
+			free(descr);
+		if (io != NULL)
+			free(io);
+		n++;
 	}
-	err = snd_output_stdio_attach(&out, stdout, 0);
-	assert(err >= 0);
-	err = snd_config_search(snd_config, "pcm", &conf);
-	if (err < 0)
-		return;
-	printf(_("PCM list:\n"));
-	snd_config_save(conf, out);
-	snd_output_close(out);
+	snd_device_name_free_hint(hints);
 }
 
 static void version(void)
 {
-	printf("%s: version " SND_UTIL_VERSION_STR " by Jaroslav Kysela <perex@suse.cz>\n", command);
+	printf("%s: version " SND_UTIL_VERSION_STR " by Jaroslav Kysela <perex@perex.cz>\n", command);
 }
 
 static void signal_handler(int sig)
@@ -319,7 +325,7 @@ static void signal_handler(int sig)
 		close(fd);
 		fd = -1;
 	}
-	if (handle) {
+	if (handle && sig != SIGABRT) {
 		snd_pcm_close(handle);
 		handle = NULL;
 	}
@@ -329,13 +335,17 @@ static void signal_handler(int sig)
 enum {
 	OPT_VERSION = 1,
 	OPT_PERIOD_SIZE,
-	OPT_BUFFER_SIZE
+	OPT_BUFFER_SIZE,
+	OPT_DISABLE_RESAMPLE,
+	OPT_DISABLE_CHANNELS,
+	OPT_DISABLE_FORMAT,
+	OPT_DISABLE_SOFTVOL
 };
 
 int main(int argc, char *argv[])
 {
 	int option_index;
-	char *short_options = "hnlLD:qt:c:f:r:d:s:MNF:A:R:T:B:vIPC";
+	char *short_options = "hnlLD:qt:c:f:r:d:MNF:A:R:T:B:vIPC";
 	static struct option long_options[] = {
 		{"help", 0, 0, 'h'},
 		{"version", 0, 0, OPT_VERSION},
@@ -349,7 +359,6 @@ int main(int argc, char *argv[])
 		{"format", 1, 0, 'f'},
 		{"rate", 1, 0, 'r'},
 		{"duration", 1, 0 ,'d'},
-		{"sleep-min", 1, 0, 's'},
 		{"mmap", 0, 0, 'M'},
 		{"nonblock", 0, 0, 'N'},
 		{"period-time", 1, 0, 'F'},
@@ -363,11 +372,15 @@ int main(int argc, char *argv[])
 		{"separate-channels", 0, 0, 'I'},
 		{"playback", 0, 0, 'P'},
 		{"capture", 0, 0, 'C'},
+		{"disable-resample", 0, 0, OPT_DISABLE_RESAMPLE},
+		{"disable-channels", 0, 0, OPT_DISABLE_CHANNELS},
+		{"disable-format", 0, 0, OPT_DISABLE_FORMAT},
+		{"disable-softvol", 0, 0, OPT_DISABLE_SOFTVOL},
 		{0, 0, 0, 0}
 	};
 	char *pcm_name = "default";
 	int tmp, err, c;
-	int do_names_list = 0, do_device_list = 0, do_pcm_list = 0;
+	int do_device_list = 0, do_pcm_list = 0;
 	snd_pcm_info_t *info;
 
 #ifdef ENABLE_NLS
@@ -408,9 +421,6 @@ int main(int argc, char *argv[])
 		case OPT_VERSION:
 			version();
 			return 0;
-		case 'n':
-			do_names_list = 1;
-			break;
 		case 'l':
 			do_device_list = 1;
 			break;
@@ -477,9 +487,6 @@ int main(int argc, char *argv[])
 		case 'd':
 			timelimit = strtol(optarg, NULL, 0);
 			break;
-		case 's':
-			sleep_min = strtol(optarg, NULL, 0);
-			break;
 		case 'N':
 			nonblock = 1;
 			open_mode |= SND_PCM_NONBLOCK;
@@ -525,24 +532,31 @@ int main(int argc, char *argv[])
 			if (file_type == FORMAT_DEFAULT)
 				file_type = FORMAT_WAVE;
 			break;
+		case OPT_DISABLE_RESAMPLE:
+			open_mode |= SND_PCM_NO_AUTO_RESAMPLE;
+			break;
+		case OPT_DISABLE_CHANNELS:
+			open_mode |= SND_PCM_NO_AUTO_CHANNELS;
+			break;
+		case OPT_DISABLE_FORMAT:
+			open_mode |= SND_PCM_NO_AUTO_FORMAT;
+			break;
+		case OPT_DISABLE_SOFTVOL:
+			open_mode |= SND_PCM_NO_SOFTVOL;
+			break;
 		default:
 			fprintf(stderr, _("Try `%s --help' for more information.\n"), command);
 			return 1;
 		}
 	}
 
-	if (do_names_list) {
-		names_list();
-		return 0;
-	} else if (do_device_list) {
+	if (do_device_list) {
 		if (do_pcm_list) pcm_list();
 		device_list();
-		snd_config_update_free_global();
-		return 0;
+		goto __end;
 	} else if (do_pcm_list) {
 		pcm_list();
-		snd_config_update_free_global();
-		return 0;
+		goto __end;
 	}
 
 	err = snd_pcm_open(&handle, pcm_name, stream, open_mode);
@@ -613,6 +627,7 @@ int main(int argc, char *argv[])
 		putchar('\n');
 	snd_pcm_close(handle);
 	free(audiobuf);
+      __end:
 	snd_output_close(log);
 	snd_config_update_free_global();
 	return EXIT_SUCCESS;
@@ -872,7 +887,6 @@ static void set_params(void)
 	snd_pcm_uframes_t buffer_size;
 	int err;
 	size_t n;
-	snd_pcm_uframes_t xfer_align;
 	unsigned int rate;
 	snd_pcm_uframes_t start_threshold, stop_threshold;
 	snd_pcm_hw_params_alloca(&params);
@@ -974,16 +988,6 @@ static void set_params(void)
 		exit(EXIT_FAILURE);
 	}
 	snd_pcm_sw_params_current(handle, swparams);
-	err = snd_pcm_sw_params_get_xfer_align(swparams, &xfer_align);
-	if (err < 0) {
-		error(_("Unable to obtain xfer align\n"));
-		exit(EXIT_FAILURE);
-	}
-	if (sleep_min)
-		xfer_align = 1;
-	err = snd_pcm_sw_params_set_sleep_min(handle, swparams,
-					      sleep_min);
-	assert(err >= 0);
 	if (avail_min < 0)
 		n = chunk_size;
 	else
@@ -991,7 +995,7 @@ static void set_params(void)
 	err = snd_pcm_sw_params_set_avail_min(handle, swparams, n);
 
 	/* round up to closest transfer boundary */
-	n = (buffer_size / xfer_align) * xfer_align;
+	n = buffer_size;
 	if (start_delay <= 0) {
 		start_threshold = n + (double) rate * start_delay / 1000000;
 	} else
@@ -1009,9 +1013,6 @@ static void set_params(void)
 	err = snd_pcm_sw_params_set_stop_threshold(handle, swparams, stop_threshold);
 	assert(err >= 0);
 
-	err = snd_pcm_sw_params_set_xfer_align(handle, swparams, xfer_align);
-	assert(err >= 0);
-
 	if (snd_pcm_sw_params(handle, swparams) < 0) {
 		error(_("unable to install sw params:"));
 		snd_pcm_sw_params_dump(swparams, log);
@@ -1219,10 +1220,10 @@ static void compute_max_peak(u_char *data, size_t count)
 		putchar('\r');
 		for (val = 0; val <= perc / 2 && val < 50; val++)
 			putchar('#');
-		for (; val < maxperc / 2 && val < 50; val++)
+		for (; val <= maxperc / 2 && val < 50; val++)
 			putchar(' ');
 		putchar('+');
-		for (++val; val < 50; val++)
+		for (++val; val <= 50; val++)
 			putchar(' ');
 
 		printf("| %02i%%", maxperc);
@@ -1239,6 +1240,7 @@ static void compute_max_peak(u_char *data, size_t count)
 			else
 				putchar(' ');
 		printf(" %i%%\n", perc);
+		fflush(stdout);
 	}
 }
 
@@ -1251,8 +1253,7 @@ static ssize_t pcm_write(u_char *data, size_t count)
 	ssize_t r;
 	ssize_t result = 0;
 
-	if (sleep_min == 0 &&
-	    count < chunk_size) {
+	if (count < chunk_size) {
 		snd_pcm_format_set_silence(hwparams.format, data + count * bits_per_frame / 8, (chunk_size - count) * hwparams.channels);
 		count = chunk_size;
 	}
@@ -1284,8 +1285,7 @@ static ssize_t pcm_writev(u_char **data, unsigned int channels, size_t count)
 	ssize_t r;
 	size_t result = 0;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		unsigned int channel;
 		size_t offset = count;
 		size_t remaining = chunk_size - count;
@@ -1332,8 +1332,7 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 	size_t result = 0;
 	size_t count = rcount;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		count = chunk_size;
 	}
 
@@ -1366,8 +1365,7 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 	size_t result = 0;
 	size_t count = rcount;
 
-	if (sleep_min == 0 &&
-	    count != chunk_size) {
+	if (count != chunk_size) {
 		count = chunk_size;
 	}
 
@@ -1454,17 +1452,15 @@ static void voc_pcm_flush(void)
 {
 	if (buffer_pos > 0) {
 		size_t b;
-		if (sleep_min == 0) {
-			if (snd_pcm_format_set_silence(hwparams.format, audiobuf + buffer_pos, chunk_bytes - buffer_pos * 8 / bits_per_sample) < 0)
-				fprintf(stderr, _("voc_pcm_flush - silence error"));
-			b = chunk_size;
-		} else {
-			b = buffer_pos * 8 / bits_per_frame;
-		}
+		if (snd_pcm_format_set_silence(hwparams.format, audiobuf + buffer_pos, chunk_bytes - buffer_pos * 8 / bits_per_sample) < 0)
+			fprintf(stderr, _("voc_pcm_flush - silence error"));
+		b = chunk_size;
 		if (pcm_write(audiobuf, b) != (ssize_t)b)
 			error(_("voc_pcm_flush error"));
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 static void voc_play(int fd, int ofs, char *name)
@@ -1981,7 +1977,7 @@ void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 			if (r == 0)
 				break;
 			l += r;
-		} while (sleep_min == 0 && (size_t)l < chunk_bytes);
+		} while ((size_t)l < chunk_bytes);
 		l = l * 8 / bits_per_frame;
 		r = pcm_write(audiobuf, l);
 		if (r != l)
@@ -1990,7 +1986,9 @@ void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 		written += r;
 		l = 0;
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 
@@ -2221,7 +2219,7 @@ void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count,
 			if (r == 0)
 				break;
 			c += r;
-		} while (sleep_min == 0 && c < expected);
+		} while (c < expected);
 		c = c * 8 / bits_per_sample;
 		r = pcm_writev(bufs, channels, c);
 		if ((size_t)r != c)
@@ -2229,7 +2227,9 @@ void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count,
 		r = r * bits_per_frame / 8;
 		count -= r;
 	}
+	snd_pcm_nonblock(handle, 0);
 	snd_pcm_drain(handle);
+	snd_pcm_nonblock(handle, nonblock);
 }
 
 void capturev_go(int* fds, unsigned int channels, off64_t count, int rtype, char **names)
